package com.ruoyi.generator.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.user.ProjectException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.generator.domain.Community.CommunityUserBlock;
import com.ruoyi.generator.mapper.community.CommunityUserBlockMapper;
import com.ruoyi.system.domain.CommunityChatMsg;
import com.ruoyi.system.domain.vo.CommunityChatMsgVo;
import com.ruoyi.system.domain.vo.SysUserVo;
import com.ruoyi.system.mapper.CommunityChatMsgMapper;
import com.ruoyi.system.mapper.SysUserMapper;
import com.ruoyi.system.service.ICommunityChatMsgService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 聊天记录
 *
 * @author fangzhen
 */
@Api(tags = "聊天记录")
@RestController
@RequestMapping("/community/chat")
public class CommunityChatMsgController extends BaseController {

    @Resource
    private ICommunityChatMsgService communityChatMsgService;

    @Resource
    private SysUserMapper sysUserMapper;

    @Resource
    private CommunityUserBlockMapper communityUserBlockMapper;

    @Resource
    private CommunityChatMsgMapper communitychatMsgMapper;
    /**
     * 获取当前登录用户聊天记录
     */
    @ApiOperation("获取当前登录用户聊天记录")
    @GetMapping()
    public AjaxResult chatRecord(Long otherUserId, String type) {
        Long userId = SecurityUtils.getUserId();
        List<CommunityChatMsgVo> chatMsgVos = new ArrayList<>();
        try {
            List<CommunityUserBlock> communityUserBlocks = communityUserBlockMapper.selectList(new QueryWrapper<CommunityUserBlock>()
                    .eq("user_id", userId)
                    .eq("peer_id", otherUserId));
            for (CommunityUserBlock userBlock : communityUserBlocks) {
                if (userBlock.isBlock()) {
                    return AjaxResult.success("当前用户已经被拉黑");
                }
            }

            /*List<CommunityChatMsg> chatMsgList = communityChatMsgService.list(new QueryWrapper<CommunityChatMsg>()
                    .and(wrapper -> wrapper.eq("sender_id", userId)
                            .eq("receiver_id", otherUserId)
                            .or()
                            .eq("sender_id", otherUserId)
                            .eq("receiver_id", userId)
                            .eq("type", type))
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", 1).or().isNull("is_delete");
                    })
                    .orderByDesc("create_time")
            );*/
            List<CommunityChatMsg> chatMsgList = communitychatMsgMapper.getChatMsgRecord(userId, otherUserId, type);

            //更新该消息标记为已读
            List<Long> msgId = chatMsgList.stream()
                    .filter(item -> !item.isReceiverIsRead() && Objects.equals(item.getReceiverId(), SecurityUtils.getUserId()))
                    .map(CommunityChatMsg::getId)
                    .collect(Collectors.toList());
            if (!msgId.isEmpty()) {
                communityChatMsgService.update(new UpdateWrapper<CommunityChatMsg>().in("id", msgId).set("receiver_is_read", true));
            }


            CommunityChatMsgVo chatMsgVo = null;
            for (CommunityChatMsg chatMsg : chatMsgList) {
                chatMsgVo = new CommunityChatMsgVo();
                BeanUtils.copyProperties(chatMsg, chatMsgVo);
                SysUser senderUser = sysUserMapper.selectUserById(chatMsgVo.getSenderId());
                SysUser receiveUser = sysUserMapper.selectUserById(chatMsgVo.getReceiverId());
                if (!msgId.isEmpty()) {
                    boolean contains = msgId.contains(chatMsg.getId());
                    if (contains) {
                        chatMsgVo.setReceiverIsRead(true);
                    }
                }
                chatMsgVo.setSenderNickName(senderUser.getNickName());
                chatMsgVo.setSenderAvatar(senderUser.getAvatar());
                chatMsgVo.setReceiverNickName(receiveUser.getNickName());
                chatMsgVo.setReceiverAvatar(receiveUser.getAvatar());
                chatMsgVos.add(chatMsgVo);
            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }


        return AjaxResult.success(chatMsgVos);
    }

    /**
     * 获取当前登录用户聊天记录
     */
    @ApiOperation("获取当前登录用户聊天列表")
    @GetMapping("/list")
    public AjaxResult chatList(String type) {
        Long userId = SecurityUtils.getUserId();
        List<SysUserVo> sysUserChatVos = null;
        try {
            sysUserChatVos = communityChatMsgService.getChatListWithLatestMessage(userId, type);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success(sysUserChatVos);
    }

    /**
     * 我的私信是否有未读
     */
    @ApiOperation("我的私信消息提示")
    @GetMapping("/messageAlert")
    public AjaxResult messageAlert(String type) {
        Long userId = SecurityUtils.getUserId();
        List<SysUserVo> sysUserChatVos = null;
        boolean hasUnreadMessages = false;
        try {
            sysUserChatVos = communityChatMsgService.getChatListWithLatestMessage(userId, type);
            // 判断是否有未读消息
            hasUnreadMessages = sysUserChatVos.stream()
                    .anyMatch(vo -> Objects.equals(vo.getReceiverId(), userId) && !vo.isReceiverIsRead());
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success(hasUnreadMessages);
    }

    /**
     * 清空聊天记录
     */
    @ApiOperation("清空聊天记录")
    @PostMapping("/clearChat")
    public AjaxResult clearChat(Long otherUserId) {
        Long userId = SecurityUtils.getUserId();
        try {
            communityChatMsgService.update(
                    null,
                    new UpdateWrapper<CommunityChatMsg>()
                            .eq("sender_id", userId)
                            .eq("receiver_id", otherUserId)
                            .set("is_delete", true)
                            .and((wrapper) -> {
                                wrapper.ne("is_delete", 1).or().isNull("is_delete");
                            })
            );
            communityChatMsgService.update(
                    null,
                    new UpdateWrapper<CommunityChatMsg>()
                            .eq("sender_id", otherUserId )
                            .eq("receiver_id", userId)
                            .set("receiver_is_delete", true)
                            .and((wrapper) -> {
                                wrapper.ne("receiver_is_delete", 1).or().isNull("receiver_is_delete");
                            })
            );
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success("清空聊天记录成功");
    }
}
